@@ -1,6 +1,6 @@
 /*
    Copyright (C) 2004 - 2013 by Guillaume Melquiond <guillaume.melquiond@inria.fr>
-   Part of the Gappa tool http://gappa.gforge.inria.fr/
+   Part of the Gappa tool https://gappa.gitlabpages.inria.fr/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the CeCILL Free Software License Agreement or
@@ -18,9 +18,10 @@ undefined_map const *instances;
 typedef std::map< std::string, backend * > backend_map;
 static static_ptr<backend_map> backends;
 
-backend::backend(std::string const &name)
+backend::backend(std::string const &n, std::string const &d)
+  : doc(d)
 {
-  (*backends)[name] = this;
+  (*backends)[n] = this;
 }
 
 backend *backend::find(std::string const &name) {
@@ -34,3 +35,11 @@ std::string composite(char prefix, int num) {
   s << prefix << (num < 0 ? -num : num);
   return s.str();
 }
+
+void backend::usage(std::ostream &out)
+{
+  for (auto const &b: *backends) {
+    out << "  -B" << b.first << std::string(30 - b.first.size(), ' ')
+        << b.second->doc << '\n';
+  }
+}
