@@ -1,6 +1,6 @@
 /*
    Copyright (C) 2004 - 2016 by Guillaume Melquiond <guillaume.melquiond@inria.fr>
-   Part of the Gappa tool http://gappa.gforge.inria.fr/
+   Part of the Gappa tool https://gappa.gitlabpages.inria.fr/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the CeCILL Free Software License Agreement or
@@ -9,12 +9,10 @@
    See the COPYING and COPYING.GPL files for more details.
 */
 
-#include <algorithm>
 #include <cassert>
 #include <cstdint>
 #include <iostream>
 #include <map>
-#include <sstream>
 #include <unordered_map>
 
 #include <gmp.h>
@@ -65,9 +63,9 @@ typedef std::map< ast_real const *, int > product;
 struct product_hash {
   std::size_t operator()(product const &p) const {
     std::size_t h = 0x12345678;
-    for (product::const_iterator i = p.begin(), i_end = p.end(); i != i_end; ++i) {
-      uintptr_t p = reinterpret_cast<uintptr_t>(i->first);
-      h = h * 5 + (std::size_t)p + (std::size_t)i->second;
+    for (auto const &i: p) {
+      uintptr_t p = reinterpret_cast<uintptr_t>(i.first);
+      h = h * 5 + (std::size_t)p + (std::size_t)i.second;
     }
     return h;
   }
@@ -80,8 +78,8 @@ typedef std::pair< sum, sum > quotient;
 static void dump_sum(std::ostream &res, sum const &s)
 {
   bool first_term = true;
-  for(sum::const_iterator i = s.begin(), i_end = s.end(); i != i_end; ++i) {
-    mpz_class coef = i->second;
+  for (auto const &i: s) {
+    mpz_class coef = i.second;
     if (coef < 0) {
       coef = -coef;
       if (first_term) { res << '-'; first_term = false; }
@@ -90,14 +88,14 @@ static void dump_sum(std::ostream &res, sum const &s)
       if (first_term) first_term = false;
       else res << " + ";
     }
-    if (i->first.empty()) res << coef;
+    if (i.first.empty()) res << coef;
     else if (coef != 1) res << coef << " * ";
     bool first_factor = true;
-    for(product::const_iterator j = i->first.begin(), j_end = i->first.end(); j != j_end; ++j) {
+    for (auto const &j: i.first) {
       if (first_factor) first_factor = false;
       else res << " * ";
-      res << '(' << dump_real(j->first) << ')';
-      if (j->second != 1) res << '^' << j->second;
+      res << '(' << dump_real(j.first) << ')';
+      if (j.second != 1) res << '^' << j.second;
     }
   }
 }
@@ -147,21 +145,21 @@ static void add_factor(sum &res, product const &factor, mpz_class const &coef)
 
 static sum neg(sum const &s) {
   sum res;
-  for(sum::const_iterator i = s.begin(), end = s.end(); i != end; ++i)
-    res.insert(std::make_pair(i->first, -i->second));
+  for (auto const &i: s)
+    res.insert(std::make_pair(i.first, -i.second));
   return res;
 }
 
 static void fma(sum &res, sum const &s1, sum const &s2) {
-  for(sum::const_iterator i = s1.begin(), i_end = s1.end(); i != i_end; ++i)
-    for(sum::const_iterator j = s2.begin(), j_end = s2.end(); j != j_end; ++j)
-      add_factor(res, mul(i->first, j->first), i->second * j->second);
+  for (auto const &i: s1)
+    for (auto const &j: s2)
+      add_factor(res, mul(i.first, j.first), i.second * j.second);
 }
 
 static void fnma(sum &res, sum const &s1, sum const &s2) {
-  for(sum::const_iterator i = s1.begin(), i_end = s1.end(); i != i_end; ++i)
-    for(sum::const_iterator j = s2.begin(), j_end = s2.end(); j != j_end; ++j)
-      add_factor(res, mul(i->first, j->first), - i->second * j->second);
+  for (auto const &i: s1)
+    for (auto const &j: s2)
+      add_factor(res, mul(i.first, j.first), - i.second * j.second);
 }
 
 static quotient neg(quotient const &q) {
@@ -170,14 +168,14 @@ static quotient neg(quotient const &q) {
 
 static void gcd(mpz_t fact, sum const &s)
 {
-  for (sum::const_iterator i = s.begin(), i_end = s.end(); i != i_end; ++i)
-    mpz_gcd(fact, fact, i->second.value);
+  for (auto const &i: s)
+    mpz_gcd(fact, fact, i.second.value);
 }
 
 static void divide(sum &s, mpz_t const fact)
 {
-  for (sum::iterator i = s.begin(), i_end = s.end(); i != i_end; ++i)
-    mpz_divexact(i->second.value, i->second.value, fact);
+  for (auto &i: s)
+    mpz_divexact(i.second.value, i.second.value, fact);
 }
 
 static void reduce(quotient &q)
