@@ -1,6 +1,6 @@
 /*
    Copyright (C) 2004 - 2015 by Guillaume Melquiond <guillaume.melquiond@inria.fr>
-   Part of the Gappa tool http://gappa.gforge.inria.fr/
+   Part of the Gappa tool https://gappa.gitlabpages.inria.fr/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the CeCILL Free Software License Agreement or
@@ -9,9 +9,9 @@
    See the COPYING and COPYING.GPL files for more details.
 */
 
-#include <algorithm>
 #include <iostream>
 #include <stack>
+
 #include "numbers/interval_utility.hpp"
 #include "numbers/real.hpp"
 #include "numbers/round.hpp"
@@ -46,7 +46,7 @@ struct splitter
    * @return false if all the intervals have been handled.
    */
   virtual number next(int &iter_max) = 0;
-  virtual ~splitter() {}
+  virtual ~splitter() = default;
   /**
    * Indicates whether intervals should be merged afterwards.
    * @return false by default.
@@ -474,9 +474,8 @@ void graph_t::dichotomize(dichotomy_hint const &hint, int iter_max)
   {
     if (g->contradiction) continue;
     only_contradictions = false;
-    for (node_map::const_iterator j = g->known_reals.begin(),
-         j_end = g->known_reals.end(); j != j_end; ++j)
-      reals.insert(j->first);
+    for (auto const &j: g->known_reals)
+      reals.insert(j.first);
     break;
   }
   assert(!contradiction);
