@@ -1,6 +1,6 @@
 /*
    Copyright (C) 2004 - 2016 by Guillaume Melquiond <guillaume.melquiond@inria.fr>
-   Part of the Gappa tool http://gappa.gforge.inria.fr/
+   Part of the Gappa tool https://gappa.gitlabpages.inria.fr/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the CeCILL Free Software License Agreement or
@@ -9,8 +9,10 @@
    See the COPYING and COPYING.GPL files for more details.
 */
 
+#include <algorithm>
 #include <iostream>
 #include <list>
+
 #include "numbers/interval_utility.hpp"
 #include "numbers/real.hpp"
 #include "parser/ast.hpp"
@@ -157,10 +159,8 @@ void dependent_node::clean_dependencies()
 void dependent_node::subst_subproof(node *m, node *n)
 {
   bool found = false;
-  for (node_vect::iterator i = pred.begin(),
-       i_end = pred.end(); i != i_end; ++i)
-  {
-    if (*i == m) { *i = n; found = true; }
+  for (node *&i: pred) {
+    if (i == m) { i = n; found = true; }
   }
   assert(found);
   n->succ.insert(this);
@@ -320,7 +320,7 @@ intersection_node::intersection_node(node *n1, node *n2)
   insert_pred(n2);
   if (is_empty(res.bnd()))
   {
-    if (res1.real.pred() == PRED_REL)
+    if (res1.real.pred() == PRED_REL || res1.real.pred() == PRED_LIN)
     {
       // "always 0" is not a contradiction, so state that the real is "0" instead
       res = property(res1.real.real2(), interval(0, 0));
@@ -369,10 +369,8 @@ graph_t::graph_t(graph_t *f, property_tree const &h)
   {
     assert(!f->contradiction);
     known_reals = f->known_reals;
-    for (node_map::iterator i = known_reals.begin(),
-         i_end = known_reals.end(); i != i_end; ++i)
-    {
-      ++i->second->nb_good;
+    for (auto const &i: known_reals) {
+      ++i.second->nb_good;
     }
     for (logic_node *n: f->trees) {
       ++n->nb_good;
@@ -389,10 +387,8 @@ graph_t::graph_t(graph_t *f, property_tree const &h)
  */
 void graph_t::purge()
 {
-  for (node_map::const_iterator i = known_reals.begin(),
-       i_end = known_reals.end(); i != i_end; ++i)
-  {
-    i->second->remove_known();
+  for (auto const &i: known_reals) {
+    i.second->remove_known();
   }
   known_reals.clear();
   for (logic_node *n: trees) {
@@ -541,8 +537,8 @@ void graph_t::replace_known(node_vect const &v)
     ++n->nb_good;
     known_reals.insert(std::make_pair(n->get_result().real, n));
   }
-  for (node_map::const_iterator i = old.begin(), end = old.end(); i != end; ++i)
-    i->second->remove_known();
+  for (auto const &i: old)
+    i.second->remove_known();
 }
 
 /**
