@@ -1,6 +1,6 @@
 /*
-   Copyright (C) 2004 - 2015 by Guillaume Melquiond <guillaume.melquiond@inria.fr>
-   Part of the Gappa tool http://gappa.gforge.inria.fr/
+   Copyright (C) 2004 - 2025 by Guillaume Melquiond <guillaume.melquiond@inria.fr>
+   Part of the Gappa tool https://gappa.gitlabpages.inria.fr/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the CeCILL Free Software License Agreement or
@@ -9,16 +9,15 @@
    See the COPYING and COPYING.GPL files for more details.
 */
 
-#include <iostream>
 #include <boost/preprocessor/cat.hpp>
 #include "backends/backend.hpp"
+
 #include "utils.hpp"
 #include "numbers/real.hpp"
 #include "numbers/interval.hpp"
 #include "numbers/interval_utility.hpp"
 #include "parser/ast.hpp"
 #include "proofs/rewriting.hpp"
-#include "proofs/updater.hpp"
 
 extern backend *proof_generator;
 extern bool parameter_constrained;
@@ -41,19 +40,18 @@ struct rewriting_scheme: proof_scheme
 {
   predicated_real rewritten;
   pattern_cond_vect conditions;
-  preal_vect needed_reals(predicated_real const &, pattern_cond_vect const &) const;
+  static preal_vect needed_reals(predicated_real const &, pattern_cond_vect const &);
   rewriting_scheme(predicated_real const &src, predicated_real const &dst, char const *n, pattern_cond_vect const &c)
     : proof_scheme(src, needed_reals(dst, c), n, UPD_COPY), rewritten(dst), conditions(c) {}
   virtual void compute(property const[], property &, std::string &) const;
 };
 
-preal_vect rewriting_scheme::needed_reals(predicated_real const &d,
-  pattern_cond_vect const &c) const
+preal_vect rewriting_scheme::needed_reals(predicated_real const &d, pattern_cond_vect const &c)
 {
   preal_vect res;
-  for (pattern_cond_vect::const_iterator i = c.begin(), i_end = c.end();
-       i != i_end; ++i)
-    res.push_back(predicated_real(i->real, i->type == COND_NZ ? PRED_NZR : PRED_BND));
+  for (pattern_cond const &i: c) {
+    res.push_back(predicated_real(i.real, i.type == COND_NZ ? PRED_NZR : PRED_BND));
+  }
   if (!d.null()) res.push_back(d);
   return res;
 }
@@ -98,16 +96,17 @@ static rewriting_scheme *generate_rewriting_scheme(predicated_real const &src,
   predicated_real const &dst, std::string const &name, rewriting_rule const *rule,
   ast_real_vect const &holders)
 {
-  for (pattern_excl_vect::const_iterator i = rule->excl.begin(),
-       i_end = rule->excl.end(); i != i_end; ++i)
-  {
-    if (rewrite(i->first, holders) == rewrite(i->second, holders))
+  if (src.real()->is_constant && !src.real2()) return NULL;
+
+  for (pattern_excl const &i: rule->excl) {
+    if (rewrite(i.first, holders) == rewrite(i.second, holders))
       return NULL;
   }
 
   pattern_cond_vect c(rule->cond);
-  for (pattern_cond_vect::iterator i = c.begin(), end = c.end(); i != end; ++i)
-    i->real = rewrite(i->real, holders);
+  for (pattern_cond &i: c) {
+    i.real = rewrite(i.real, holders);
+  }
   return new rewriting_scheme(src, dst, name.c_str(), c);
 }
 
@@ -178,12 +177,12 @@ struct proxy_rewriting_factory: scheme_factory
 proof_scheme *proxy_rewriting_factory::operator()(predicated_real const &src,
   ast_real_vect const &holders) const
 {
+  if (src.real()->is_constant && !src.real2()) return NULL;
+
   if (excl)
   {
-    for (pattern_excl_vect::const_iterator i = excl->begin(),
-         i_end = excl->end(); i != i_end; ++i)
-    {
-      if (rewrite(i->first, holders) == rewrite(i->second, holders))
+    for (pattern_excl const &i: *excl) {
+      if (rewrite(i.first, holders) == rewrite(i.second, holders))
         return NULL;
     }
   }
@@ -192,9 +191,7 @@ proof_scheme *proxy_rewriting_factory::operator()(predicated_real const &src,
   ast_real const *r = rewrite(dst.real(), holders);
   preal_vect needed;
   needed.push_back(predicated_real(rewrite(rsrc, holders), rewrite(unhide(rdst), holders), PRED_EQL));
-  needed.push_back(t == PRED_REL
-    ? predicated_real(r, rewrite(dst.real2(), holders), t)
-    : predicated_real(r, t));
+  needed.push_back(predicated_real(r, dst.real2() ? rewrite(dst.real2(), holders) : NULL, t));
   return new proxy_rewriting_scheme(src, needed, name.c_str());
 }
 
@@ -217,6 +214,10 @@ static void generate_all_proxy_rewrite(ast_real const *src,
     pr(dst, free, PRED_REL), "rel_rewrite_1", e);
   new proxy_rewriting_factory(src, dst, pr(free, src, PRED_REL),
     pr(free, dst, PRED_REL), "rel_rewrite_2", e);
+  new proxy_rewriting_factory(src, dst, pr(src, free, PRED_LIN),
+    pr(dst, free, PRED_LIN), "lin_rewrite_1", e);
+  new proxy_rewriting_factory(src, dst, pr(free, src, PRED_LIN),
+    pr(free, dst, PRED_LIN), "lin_rewrite_2", e);
 }
 
 RUN_ONCE(approximate_rewrite) {
@@ -228,10 +229,8 @@ void register_user_rewrite(ast_real const *r1, ast_real const *r2, hint_cond_vec
   pattern_cond_vect pc;
   if (hc)
   {
-    for (hint_cond_vect::const_iterator i = hc->begin(),
-         i_end = hc->end(); i != i_end; ++i)
+    for (hint_cond const *c: *hc)
     {
-      hint_cond const *c = *i;
       pattern_cond cond;
       cond.real = c->real;
       cond.value = atoi(c->value->mantissa.c_str());
@@ -273,7 +272,7 @@ pattern_excl_vect operator&&(pattern_excl_vect const &v, pattern_excl const &c)
   return res;
 }
 
-static pattern a(0), b(1), c(2), d(3), a_b(-1), one(token_one);
+static pattern a(0), b(1), c(2), d(3), e(4), a_b(-1), one(token_one);
 
 #define abs pattern::abs
 #define sqrt pattern::sqrt
@@ -283,10 +282,11 @@ static pattern a(0), b(1), c(2), d(3), a_b(-1), one(token_one);
 
 /*
 Naming convention: operator name followed by
-  x (expand term)      f (factor term)      m (mix)
-  a (approximate expr) e (accurate expr)    i (irrelevant)
-  l (left hand side)   r (right hand side)  b (both)
-  s (absolute error)   q (relative error)   e (equality)   u (user term)
+- x (expand term), f (factor term), m (mix)
+- a (approximate expr), e (accurate expr), i (irrelevant)
+- l (left hand side), r (right hand side), b (both), s t (skewed)
+- s (absolute error), q (relative error), m (relative magnitude),
+  e (equality), u (user term)
 */
 
 #define REWRITE(name,p1,p2) \
@@ -351,18 +351,6 @@ REWRITe(add_firs,
 	a - c,
 	a ^ c);
 
-REWRIT9(add_filq,
-	((a + b) - (a + c)) / (a + c),
-	(b - c) / (a + c),
-	~(a + c),
-	b ^ c);
-
-REWRIT9(add_firq,
-	((a + b) - (c + b)) / (c + b),
-	(a - c) / (c + b),
-	~(c + b),
-	a ^ c);
-
 R3WRITE(add_xilu,
 	a,
 	hide((a + b) - b),
@@ -400,18 +388,6 @@ REWRITe(sub_firs,
 	a - c,
 	a ^ c);
 
-REWRIT9(sub_filq,
-	((a - b) - (a - c)) / (a - c),
-	hide(-((b - c) / (a - c))),
-	~(a - c),
-	b ^ c);
-
-REWRIT9(sub_firq,
-	((a - b) - (c - b)) / (c - b),
-	(a - c) / (c - b),
-	~(c - b),
-	a ^ c);
-
 R3WRITE(val_xabs, // actually sub_xilu
 	b,
 	hide(a + (b - a)),
@@ -476,12 +452,6 @@ R3WRIT3(mul_xiru,
 
 // DIV
 
-REWRIT9(div_xals,
-	b / c,
-	hide((b - a) / c + a / c),
-	~c,
-	a ^ c && b ^ c);
-
 REWRIT3(div_fir,
 	(a * b) / b,
 	a,
@@ -492,12 +462,6 @@ REWRIT3(div_fil,
 	b,
 	~a);
 
-REWRIT9(div_firs,
-	a / b - c / b,
-	hide((a - c) / b),
-	~b,
-	a ^ c);
-
 R3WRIT3(div_xilu,
 	a,
 	hide((a / b) * b),
@@ -508,7 +472,7 @@ R3WRIT3(div_xiru,
 	b,
 	hide(a / (a / b)),
 	a / b,
-	~a && ~b);
+	~a);
 
 // SQRT
 
@@ -570,74 +534,35 @@ REWRIT3(square_sqrt,
 	a,
 	a >= 0);
 
-REWRIT3(addf_1,
+REWRIT3(addf_9,
 	a / (a + b),
-	hide(one / hide(one + b / a)),
+	hide(one - hide(one / hide(one + a / b))),
+	~b && ~(a + b));
+
+REWRIT3(addf_10,
+	b / (a + b),
+	hide(one - hide(one / hide(one + b / a))),
 	~a && ~(a + b));
 
-REWRIT9(addf_2,
-	a / (a + b),
-	hide(one - b / (a + b)),
-	~(a + b),
-	a ^ b);
-
-REWRIT9(addf_3,
+REWRIT3(addf_11,
 	a / (a - b),
-	hide(one / hide(one - b / a)),
-	~a && ~(a - b),
-	a ^ b);
+	hide(one - hide(one / hide(one - a / b))),
+	~b && ~(a - b));
 
-REWRIT9(addf_4,
-	a / (a - b),
-	hide(one + b / (a - b)),
-	~(a - b),
-	a ^ b);
-
-REWRIT9(addf_5,
-	b / (a + b),
-	hide(one / hide(a / b + one)),
-	~b && ~(a + b),
-	a ^ b);
-
-REWRIT9(addf_6,
-	b / (a + b),
-	hide(one - a / (a + b)),
-	~(a + b),
-	a ^ b);
-
-REWRIT9(addf_7,
+REWRIT3(addf_12,
 	b / (a - b),
-	hide(one / hide(a / b - one)),
-	~b && ~(a - b),
-	a ^ b);
-
-REWRIT9(addf_8,
-	b / (a - b),
-	hide(a / (a - b) - one),
-	~(a - b),
-	a ^ b);
+	hide(hide(one / hide(one - b / a)) - one),
+	~a && ~(a - b));
 
 #undef REWRITE
 #undef REWRIT3
 #undef REWRITe
 #undef REWRIT9
 
-#define REWRITE(name,lhs1,rhs1,lhs2,rhs2) \
-  static rewriting_rule REWRITING_NAME \
-    (predicated_real(lhs1, rhs1, PRED_REL), predicated_real(lhs2, rhs2, PRED_REL),\
-     #name, pattern_cond_vect(), pattern_excl_vect(), NULL)
-#define REWRIT3(name,lhs1,rhs1,lhs2,rhs2,cond) \
-  static rewriting_rule REWRITING_NAME \
-    (predicated_real(lhs1, rhs1, PRED_REL), predicated_real(lhs2, rhs2, PRED_REL),\
-     #name, pattern_cond_vect() && cond, pattern_excl_vect(), NULL)
 #define REWRITe(name,lhs1,rhs1,lhs2,rhs2,excl) \
   static rewriting_rule REWRITING_NAME \
     (predicated_real(lhs1, rhs1, PRED_REL), predicated_real(lhs2, rhs2, PRED_REL),\
      #name, pattern_cond_vect(), pattern_excl_vect() && excl, NULL)
-#define REWRIT9(name,lhs1,rhs1,lhs2,rhs2,cond,excl) \
-  static rewriting_rule REWRITING_NAME \
-    (predicated_real(lhs1, rhs1, PRED_REL), predicated_real(lhs2, rhs2, PRED_REL),\
-     #name, pattern_cond_vect() && cond, pattern_excl_vect() && excl, NULL)
 
 REWRITe(opp_fibq,
 	-a, -b,
@@ -659,27 +584,92 @@ REWRITe(div_firq,
 	a, c,
 	a ^ c);
 
-#undef REWRITE
-#undef REWRIT3
 #undef REWRITe
-#undef REWRIT9
 
 #define REWRITE(name,lhs1,rhs1,lhs2,rhs2) \
   static rewriting_rule REWRITING_NAME \
-    (predicated_real(lhs1, rhs1, PRED_EQL), predicated_real(lhs2, rhs2, PRED_EQL),\
+    (predicated_real(lhs1, rhs1, PRED_LIN), predicated_real(lhs2, rhs2, PRED_LIN),\
      #name, pattern_cond_vect(), pattern_excl_vect(), NULL)
-#define REWRIT3(name,lhs1,rhs1,lhs2,rhs2,cond) \
+#define REWRITe(name,lhs1,rhs1,lhs2,rhs2,excl) \
   static rewriting_rule REWRITING_NAME \
-    (predicated_real(lhs1, rhs1, PRED_EQL), predicated_real(lhs2, rhs2, PRED_EQL),\
-     #name, pattern_cond_vect() && cond, pattern_excl_vect(), NULL)
+    (predicated_real(lhs1, rhs1, PRED_LIN), predicated_real(lhs2, rhs2, PRED_LIN),\
+     #name, pattern_cond_vect(), pattern_excl_vect() && excl, NULL)
+
+REWRITe(opp_fibm,
+	-a, -b,
+	a, b,
+	a ^ b);
+
+REWRITe(mul_film,
+	a * b, a * c,
+	b, c,
+	b ^ c);
+
+REWRITe(mul_firm,
+	a * b, c * a,
+	b, c,
+	a ^ c);
+
+REWRITE(mul_fism,
+	a * b, a * c,
+	b, c);
+
+REWRITE(mul_fitm,
+	a * b, b * c,
+	a, c);
+
+REWRITe(div_firm,
+	a / b, c / b,
+	a, c,
+	a ^ c);
+
+REWRITe(val_xabm,
+	b, c,
+	hide(a + (b - a)), c,
+	a ^ c && b ^ c);
+
+REWRITe(add_mibm,
+	(a + b) - (c + d), e,
+	hide((a - c) + (b - d)), e,
+	a ^ c && b ^ d);
+
+REWRITe(add_film,
+	(a + b) - (a + c), d,
+	b - c, d,
+	b ^ c);
+
+REWRITe(add_firm,
+	(a + b) - (c + b), d,
+	a - c, d,
+	a ^ c);
+
+REWRITe(sub_mibm,
+	(a - b) - (c - d), e,
+	hide((a - c) - (b - d)), e,
+	a ^ c && b ^ d);
+
+REWRITe(sub_film,
+	(a - b) - (a - c), e,
+	hide(-(b - c)), e,
+	b ^ c);
+
+REWRITe(sub_firm,
+	(a - b) - (c - b), e,
+	a - c, e,
+	a ^ c);
+
+REWRITe(sub_xalm,
+	b - c, d,
+	hide((b - a) + (a - c)), d,
+	a ^ c && b ^ c);
+
+#undef REWRITE
+#undef REWRITe
+
 #define REWRITe(name,lhs1,rhs1,lhs2,rhs2,excl) \
   static rewriting_rule REWRITING_NAME \
     (predicated_real(lhs1, rhs1, PRED_EQL), predicated_real(lhs2, rhs2, PRED_EQL),\
      #name, pattern_cond_vect(), pattern_excl_vect() && excl, NULL)
-#define REWRIT9(name,lhs1,rhs1,lhs2,rhs2,cond,excl) \
-  static rewriting_rule REWRITING_NAME \
-    (predicated_real(lhs1, rhs1, PRED_EQL), predicated_real(lhs2, rhs2, PRED_EQL),\
-     #name, pattern_cond_vect() && cond, pattern_excl_vect() && excl, NULL)
 
 REWRITe(opp_fibe,
 	-a, -b,
@@ -725,3 +715,5 @@ REWRITe(div_fire,
 	a / b, c / b,
 	a, c,
 	a ^ c);
+
+#undef REWRITe
