@@ -1,6 +1,6 @@
 /*
    Copyright (C) 2004 - 2015 by Guillaume Melquiond <guillaume.melquiond@inria.fr>
-   Part of the Gappa tool http://gappa.gforge.inria.fr/
+   Part of the Gappa tool https://gappa.gitlabpages.inria.fr/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the CeCILL Free Software License Agreement or
@@ -314,8 +314,8 @@ static void mark_useful_reals(predicated_real const &real)
     {
       if (!s) continue;
       preal_vect w = s->needed_reals;
-      for (preal_vect::const_iterator j = w.begin(), j_end = w.end(); j != j_end; ++j)
-        if (reals[*j].can_visit()) pending_reals.push_back(*j);
+      for (predicated_real const &j: w)
+        if (reals[j].can_visit()) pending_reals.push_back(j);
     }
   }
 }
@@ -573,11 +573,9 @@ void graph_t::populate(property_tree const &targets,
     scheme_queue missing_schemes;
     initialize_scheme_list(missing_schemes);
     int iter = 0;
-    for (node_map::const_iterator i = known_reals.begin(),
-         i_end = known_reals.end(); i != i_end; ++i)
-    {
-      if (!is_usable(i->second)) continue;
-      insert_dependent(missing_schemes, i->first);
+    for (auto const &i: known_reals) {
+      if (!is_usable(i.second)) continue;
+      insert_dependent(missing_schemes, i.first);
       --iter;
     }
     proof_scheme const *s;
@@ -623,11 +621,9 @@ void graph_t::populate(property_tree const &targets,
     if (dichotomy_it == dichotomy_end) break;
     dichotomize(*dichotomy_it, iter_max);
     if (contradiction) return;
-    for (node_map::const_iterator i = known_reals.begin(),
-         i_end = known_reals.end(); i != i_end; ++i)
-    {
-      if (!is_usable(i->second)) continue;
-      reduce_hypotheses(i->second, trees, NULL, NULL);
+    for (auto const &i: known_reals) {
+      if (!is_usable(i.second)) continue;
+      reduce_hypotheses(i.second, trees, NULL, NULL);
       if (contradiction) return;
     }
     if (split_hypotheses(trees, current_targets, &missing_schemes)) return;
@@ -639,22 +635,22 @@ void graph_t::populate(property_tree const &targets,
     i->tree.get_splitting(s);
   unsigned max_pts = 0;
   splitting::value_type const *sv = NULL;
-  for (splitting::const_iterator i = s.begin(), i_end = s.end(); i != i_end; ++i)
+  for (auto const &i: s)
   {
-    unsigned nb = i->second.size();
+    unsigned nb = i.second.size();
     if (nb <= max_pts) continue;
-    node *n = find_proof(predicated_real(i->first, PRED_BND));
+    node *n = find_proof(predicated_real(i.first, PRED_BND));
     if (n) {
       interval const &b = n->get_result().bnd();
       if (is_singleton(b)) continue;
       nb = 0;
-      for (split_point const &j: i->second) {
+      for (split_point const &j: i.second) {
         if (lower(b) < j.pt && j.pt < upper(b)) ++nb;
       }
       if (nb <= max_pts) continue;
     }
     max_pts = nb;
-    sv = &*i;
+    sv = &i;
   }
   if (max_pts)
   {
@@ -673,11 +669,9 @@ void graph_t::populate(property_tree const &targets,
 
   // Fill all the remaining holes in the formula.
   if (!umap) return;
-  for (node_map::const_iterator i = known_reals.begin(),
-       i_end = known_reals.end(); i != i_end; ++i)
-  {
-    if (!is_usable(i->second)) continue;
-    reduce_hypotheses(i->second, trees, NULL, umap);
+  for (auto const &i: known_reals) {
+    if (!is_usable(i.second)) continue;
+    reduce_hypotheses(i.second, trees, NULL, umap);
     if (contradiction) return;
   }
 }
